/*---------------------------------------------------------------------------------
DSaber for DS Motion Card
created by KeithE
Updated January 30, 2007
	Now uses the libnds Motion Card functions
	Now enables the DS Motion Card every frame so slot-1 card users can swap cards

	

---------------------------------------------------------------------------------*/
#include <nds.h>
#include <nds/arm9/sound.h>		// sound functions
#include <nds/arm9/ndsmotion.h>
#include <stdio.h>
#include <math.h>

// Headers generated by bin2o rule
#include "clash_raw.h"
#include "hum_raw.h"
#include "swing_raw.h"
#include "powerup_raw.h"
#include "powerdown_raw.h"

#define SAMPLERATE 22050

//---------------------------------------------------------------------------------
int main(void) {
//---------------------------------------------------------------------------------

	powerON( POWER_LCD | POWER_2D_B );

	// initialise the irq dispatcher
	irqInit();
	// a vblank interrupt is needed to use swiWaitForVBlank()
	// since the dispatcher handles the flags no handler is required
	irqSet(IRQ_VBLANK, NULL);
	irqEnable(IRQ_VBLANK);

	videoSetMode(MODE_0_2D | DISPLAY_BG0_ACTIVE);	// main screen
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);	//sub bg 0

	BG_PALETTE[0] = RGB15(0,0,0);	//black background
	BG_PALETTE_SUB[0] = RGB15(0,0,0);	//black background

	motion_enable();

	u16 keys;

	// set the generic sound parameters
	setGenericSound(	SAMPLERATE,	/* sample rate */
						127,	/* volume */
						64,		/* panning */
						0 );	/* sound format*/

	
	
	int hum_num_vblanks = hum_raw_size*30/SAMPLERATE;
	int hum_counter = 0;
	int powerup_num_vblanks = powerup_raw_size*30/SAMPLERATE;
	int powerup_counter = 0;
	int powerdown_num_vblanks = powerdown_raw_size*30/SAMPLERATE;
	int powerdown_counter = 0;
	int clash_num_vblanks = clash_raw_size*30/SAMPLERATE;
	int clash_counter = 0;
	int clash_playing = 0;
	int swing_num_vblanks = swing_raw_size*30/SAMPLERATE;
	int swing_counter = 0;
	int swing_playing = 0;
	
	int timeout = 5*60; // time until automatic shutdown
	int timeout_counter = 0;
	
	int state = 0; // 0 means off
	
	// ring buffer for storing accelerations
	int Xaccel[5] = {0};
	int Yaccel[5] = {0};
	int Zaccel[5] = {0};
	
	int accel_index = 0;
	
	for(accel_index = 0;accel_index<5;accel_index++) {
		swiWaitForVBlank();
		Xaccel[accel_index] = motion_read_x();
		Yaccel[accel_index] = motion_read_y();
		Zaccel[accel_index] = motion_read_z();
	}
	
	accel_index = 0;
	
	int Xaccel_avg = (Xaccel[0] + Xaccel[1] + Xaccel[2] + Xaccel[3] + Xaccel[4])/5;
	int Yaccel_avg = (Yaccel[0] + Yaccel[1] + Yaccel[2] + Yaccel[3] + Yaccel[4])/5;
	int Zaccel_avg = (Zaccel[0] + Zaccel[1] + Zaccel[2] + Zaccel[3] + Zaccel[4])/5;
	
	// thresholds for various actions
	int powerup_thresh = 50;
	int swing_thresh = 300;
	int clash_thresh = 1300;
	
	while(1) {

		swiWaitForVBlank();
		scanKeys();
		keys = keysDown();
		
		motion_enable(); // putting this in while loop allows the DS Motion Card to be inserted at any time
		
		Xaccel[accel_index] = motion_read_x();
		Yaccel[accel_index] = motion_read_y();
		Zaccel[accel_index] = motion_read_z();
		
		// use average acceleration over past 5 frames
		// we'll be comparing current acceleration to this running average
		int Xaccel_avg = (Xaccel[0] + Xaccel[1] + Xaccel[2] + Xaccel[3] + Xaccel[4])/5;
		int Yaccel_avg = (Yaccel[0] + Yaccel[1] + Yaccel[2] + Yaccel[3] + Yaccel[4])/5;
		int Zaccel_avg = (Zaccel[0] + Zaccel[1] + Zaccel[2] + Zaccel[3] + Zaccel[4])/5;
		
		if(state == 0) { // 0 is powered down
			if( fabs(Xaccel[accel_index]-Xaccel_avg) > powerup_thresh || fabs(Yaccel[accel_index]-Yaccel_avg) > powerup_thresh || fabs(Zaccel[accel_index]-Zaccel_avg) > powerup_thresh) {
				playGenericSound(powerup_raw, powerup_raw_size);
				state = 1; // 1 means powering up
				powerup_counter = 0;
			}
		}
		
		if(state == 1) { // 1 is powering up (playing powerup sound)
			// keep incrementing powerup_counter
			// stay in state 1 until powerup sound is done playing
			powerup_counter++;
			BG_PALETTE[0] = RGB15(0,31*powerup_counter/powerup_num_vblanks,0);	// ramp to green background
			BG_PALETTE_SUB[0] = RGB15(0,31*powerup_counter/powerup_num_vblanks,0);	// ramp to green background
			if(powerup_counter>powerup_num_vblanks) {
				state = 2;
				hum_counter = hum_num_vblanks; // do this so hum will start playing
			}
		}
		
		if(state == 2) { // 2 is powered up
			hum_counter++;
			// keep the background changing color from black to green and back
			if(hum_counter < hum_num_vblanks/2) { // ramping up
				BG_PALETTE[0] = RGB15(0,31*hum_counter/(hum_num_vblanks/2),0);	// ramp up to green background
				BG_PALETTE_SUB[0] = RGB15(0,31*hum_counter/(hum_num_vblanks/2),0);	// ramp up to green background
			}
			else {
				BG_PALETTE[0] = RGB15(0,31-31*(hum_counter-hum_num_vblanks/2)/(hum_num_vblanks/2),0);	// ramp down to black background
				BG_PALETTE_SUB[0] = RGB15(0,31-31*(hum_counter-hum_num_vblanks/2)/(hum_num_vblanks/2),0);	// ramp down to black background
			}
			// keep the hum sound playing
			if(hum_counter > hum_num_vblanks) {
				hum_counter = 0;
				playGenericSound(hum_raw, hum_raw_size);
			}
			// test if should play clash sound
			if ( (clash_playing == 0) && (fabs(Xaccel[accel_index]-Xaccel_avg) > clash_thresh || fabs(Yaccel[accel_index]-Yaccel_avg) > clash_thresh || fabs(Zaccel[accel_index]-Zaccel_avg) > clash_thresh ) ) {
				playGenericSound(clash_raw, clash_raw_size);
				clash_playing = 1;
			}
			// test if should play hum sound			
			if ( (swing_playing == 0) && (clash_playing == 0) && (fabs(Xaccel[accel_index]-Xaccel_avg) > swing_thresh || fabs(Yaccel[accel_index]-Yaccel_avg) > swing_thresh || fabs(Zaccel[accel_index]-Zaccel_avg) > swing_thresh ) ) {
				playGenericSound(swing_raw, swing_raw_size);
				swing_playing = 1;
			}
			
			if(clash_playing) {
				BG_PALETTE_SUB[0] = RGB15(31,0,0);	// red background
				clash_counter++;
				if(clash_counter > clash_num_vblanks) {
					clash_counter = 0;
					clash_playing = 0;
				}
			}
			
			if(swing_playing) {
				if(clash_playing==0) BG_PALETTE_SUB[0] = RGB15(0,0,31);	// blue background
				swing_counter++;
				if(swing_counter > swing_num_vblanks) {
					swing_counter = 0;
					swing_playing = 0;
				}
			}
			// if not moving, then increment the counter for shutting down
			if ( fabs(Xaccel[accel_index]-Xaccel_avg) < powerup_thresh && fabs(Yaccel[accel_index]-Yaccel_avg) < powerup_thresh && fabs(Zaccel[accel_index]-Zaccel_avg) < powerup_thresh ) {
				timeout_counter++;
			}
			else timeout_counter = 0;

			
			if ( timeout_counter > timeout) { // powerdown when hum is complete
				state = 3; // 3 is powering down
				powerdown_counter = 0;
				timeout_counter = 0;
			}
		}
		
		if(state == 3) { // 3 is powering down (playing powerdown sound)
			if(powerdown_counter == 0) { // if just starting, then wait for end of hum
				while(hum_counter < hum_num_vblanks) {
					// keep the background changing color from black to green and back
					if(hum_counter < hum_num_vblanks/2) { // ramping up
						BG_PALETTE[0] = RGB15(0,31*hum_counter/(hum_num_vblanks/2),0);	// ramp up to green background
						BG_PALETTE_SUB[0] = RGB15(0,31*hum_counter/(hum_num_vblanks/2),0);	// ramp up to green background
					}
					else {
						BG_PALETTE[0] = RGB15(0,31-31*(hum_counter-hum_num_vblanks/2)/(hum_num_vblanks/2),0);	// ramp down to black background
						BG_PALETTE_SUB[0] = RGB15(0,31-31*(hum_counter-hum_num_vblanks/2)/(hum_num_vblanks/2),0);	// ramp down to black background
					}
					swiWaitForVBlank();
					hum_counter ++;
				}
				// hum is done, so start playing powerdown sound
				playGenericSound(powerdown_raw, powerdown_raw_size);
			}
			BG_PALETTE[0] = RGB15(0,31-31*powerdown_counter/powerdown_num_vblanks,0);	// ramp to black background
			BG_PALETTE_SUB[0] = RGB15(0,31-31*powerdown_counter/powerdown_num_vblanks,0);	// ramp to black background
			powerdown_counter++;
			if(powerdown_counter > powerdown_num_vblanks) { // done shutting down
				state = 0;
				BG_PALETTE[0] = 0; // black
				BG_PALETTE_SUB[0] = 0; // black
			}
		}
		
		accel_index++; // increment index for accel ring buffer
		if(accel_index >= 5) accel_index = 0; 

	} // end while

	return 0;
}
